import datetime

# Create your views here.
from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_page
from rest_framework import serializers, status
from rest_framework.response import Response
from rest_framework.views import APIView
from satella.time import parse_time_string

from counting.cron import ReaperJob, DayType
from counting.models import StoryOfADay
from shares.models import Share, SHARE_FILE


class StoryOfADaySerializer(serializers.ModelSerializer):
    class Meta:
        model = StoryOfADay
        fields = ['day', 'links', 'files']


def get_history_for(date: datetime.date) -> StoryOfADay:
    dt = ReaperJob.get_day_type(date)
    if dt == DayType.DAY_WITH_HISTORY:
        return StoryOfADay.objects.get(date=date)
    else:
        # Count them ad hoc
        links, files = 0, 0
        for share in Share.objects.get_for_day(date):
            if share.times_used:
                if share.share_type == SHARE_FILE:
                    files += 1
                else:
                    links += 1
        return StoryOfADay(day=date, links=links, files=files)


class GetHistory(APIView):
    """
    Loads the history.

    You must be authorized to access this endpoint.

    This endpoint is cached for up to 5 minutes.

    The result will be a list of elements like
    {
        "day": "YYYY-MM-DD",
        "links": 0,
        "files": 1
    }
    """

    @method_decorator(cache_page(parse_time_string('5m')))
    def get(self, request):
        if not request.user.is_authenticated:
            return Response({}, status=status.HTTP_401_UNAUTHORIZED)
        items = set(StoryOfADay.objects.all())
        today = datetime.date.today()
        items.add(get_history_for(today))
        items.add(get_history_for(today - datetime.timedelta(days=1)))
        items = list(items)
        return Response(StoryOfADaySerializer(items, many=True).data)
